if (!apg) {
    var apg = {};
}

apg.settings = {};
apg.settings.key = "llSettings";

apg.dictionary = {};
apg.dictionary.key = "flewentDictionary";

apg.clearLocalStorage = function () {
    for (var i in localStorage) {
        delete localStorage[i];
    }
}

apg.settings.save = function (settings) {
    settings = JSON.stringify(settings);
    localStorage[apg.settings.key] = settings;
};

apg.settings.fetch = function () {
    //apg.clearLocalStorage();
    var settings;
    var defaults = {
        translateTo    : "es",
        convertPercent : "5",
        enabled        : true,
        minLength      : 4,
        maxDictionary  : 500,
        growthRate     : 4,
        highlighting   : 'on',
        customLangs    : {},
        wordBlackList  : new Array("a", "it", "at", "the"),
        filterType     : "black",
        siteBlackList  : new Array("mail.google.com"),
        siteWhiteList  : new Array("reddit.com", "wikipedia.org", "bbc.co.uk")
    };

    if (localStorage[apg.settings.key]) {
        settings = JSON.parse(localStorage[apg.settings.key]);
    }
    else {
        settings = {};
    }


    for (var i in defaults) {
        if (settings[i] != undefined && settings[i] != null) {
            defaults[i] = settings[i];
        }
    }

    if (settings.enabled) {

    }

    return defaults;
};

apg.blackListWord = function(word){
    var settings = apg.settings.fetch();
    settings.wordBlackList.push(word);
    apg.settings.save(settings);
}

apg.getTemplate = function(templateName, variables, callback)
{
    $.get(templateName+'.html', function(data) {
        for(i in variables){
            //console.log('replacing `'+'{'+i+'}'+'` with `'+variables[i]+'`');
            data = data.replace(  new RegExp('\{'+i+'\}',"gm") , variables[i])
        }
        callback(data);
    });

};

apg.dictionary.getKey = function (currentLanguage) {
    return apg.dictionary.key + '.' + currentLanguage;
}
apg.dictionary.getSizeKey = function (currentLanguage) {
    return 'dictionarySize_'+currentLanguage;
}

apg.dictionary.getDictionary = function (currentLanguage) {
    var key = apg.dictionary.getKey(currentLanguage),
        sizeKey = apg.dictionary.getSizeKey(currentLanguage),
        currentDictionary;

    if (apg.dictionary[key] !== undefined && apg.dictionary[key] !== false) {
        return apg.dictionary[key]
    }

    currentDictionary = localStorage[key];

    if (currentDictionary === undefined) {
        currentDictionary = JSON.stringify({});
    }

    //console.log('Parsing Dictionary');
    currentDictionary = JSON.parse(currentDictionary);

    //load dictionary in memory
    apg.dictionary[key] = currentDictionary;
    apg.dictionary[sizeKey] = Object.keys(currentDictionary).length;

    return currentDictionary;
}

apg.dictionary.getDictionarySize = function(currentLanguage) {
    var sizeKey = apg.dictionary.getSizeKey(currentLanguage),
        size = apg.dictionary[sizeKey];

    if(size === undefined || size === null) {
        console.log('Getting dictionary for size calculation.');
        var dictionary = apg.dictionary.getDictionary(currentLanguage);
        size = Object.keys(dictionary).length;
        apg.dictionary[sizeKey] = size;
    }
    return size;
}

apg.dictionary.save = function (currentDictionary, currentLanguage) {
    var key = apg.dictionary.getKey(currentLanguage),
        sizeKey = apg.dictionary.getSizeKey(currentLanguage);

    //update dictionary in memory
    apg.dictionary[key] = currentDictionary;
    apg.dictionary[sizeKey] = Object.keys(currentDictionary).length;

    //persist dictionary
    //console.log('Stringify Dictionary');
    localStorage[key] = JSON.stringify(currentDictionary);
}

apg.dictionary.remove = function (toRemove, currentLanguage) {
    var dictionary = apg.dictionary.getDictionary(currentLanguage);
    console.log('removing: '+toRemove);
    delete dictionary[toRemove];
    apg.blackListWord(toRemove);
    apg.dictionary.save(dictionary, currentLanguage);
}
apg.dictionary.add = function (data) {
    var currentLanguage = data.currentLanguage,
        wordsToAdd = data.wordsToAdd,
        settings = apg.settings.fetch(),
        key = apg.dictionary.getKey(currentLanguage),
        currentDictionary = apg.dictionary.getDictionary(currentLanguage),
        length = apg.dictionary.getDictionarySize(currentLanguage),
        force = data.force || false;

    //limit dictionary to user defined value
    if (length > settings.maxDictionary && force === false) {
        return;
    }

    for (var i in wordsToAdd) {
        if(currentDictionary[i] === undefined || force) {
            currentDictionary[i] = wordsToAdd[i];
        }
    }

    apg.dictionary.save(currentDictionary, currentLanguage);
};

apg.dictionary.translate = function (data) {
    var i,
        currentWord,
        toTranslate = data.toTranslate,
        currentLanguage = data.currentLanguage,
        currentDictionary = apg.dictionary.getDictionary(currentLanguage),
        translateCount = data.translateCount,
        noTranslateCount = data.noTranslateCount,
        amountTranslated = 0,
        skippedCount = 0,
        translations = {};

    //translate all the words we can. If we translate the requested amount, also fill the don't translate case, then break
    for (i in toTranslate) {
        currentWord = toTranslate[i];

        if (amountTranslated < translateCount) {
            if (currentDictionary[currentWord]) {
                translations[currentWord] = currentDictionary[currentWord];
                amountTranslated += 1;
            }
        }
        else if (skippedCount < noTranslateCount) {
            if(! currentDictionary[currentWord]) {
                translations[currentWord] = false;
                skippedCount += 1;
            }
        }
        else {
            //break this loop since it can be big and each iteration is a lookup in the potentially giant dictionary
            break;
        }

    }

    //if we couldn't translate enough words, fill the non translate case
    if (skippedCount < noTranslateCount) {
        for (i in toTranslate) {
            if (skippedCount < noTranslateCount) {
                if (!currentDictionary[toTranslate[i]]) {
                    translations[toTranslate[i]] = false;
                    skippedCount += 1;
                }
            }
            else {
                break;
            }
        }
    }

    return translations;
};

chrome.extension.onMessage.addListener(function (request, sender, sendResponse) {
    //console.log(request);
    if (request.saveSettings) {
        apg.settings.save(request.data);
        return;
    }
    if (request.addToDictionary) {
        apg.dictionary.add(request);
    }
    if (request.removeFromDictionary) {
        apg.dictionary.remove(request.toRemove, request.currentLanguage);
    }
    if (request.dictionaryTranslate) {
        sendResponse(apg.dictionary.translate(request));
    }
    if (request.getDictionary) {
        sendResponse(apg.dictionary.getDictionary(request.currentLanguage));
    }
    if (request.getDictionarySize) {
        sendResponse(apg.dictionary.getDictionarySize(request.currentLanguage));
    }
    if (request.saveDictionary) {
        sendResponse(apg.dictionary.save(request.dictionary, request.currentLanguage));
    }
    if (request.blackListWord) {
        return apg.blackListWord(request.word);
    }
    if (request.getTemplate) {
        apg.getTemplate(request.templateName, request.variables, function(data){
            sendResponse(data);
        });
        return true;
    }
    if (request.updateIcon) {
        chrome.browserAction.setIcon({path : request.path});
        return;
    }
    if (request.getSettings) {
        sendResponse(apg.settings.fetch());
        return;
    }
    if (request.newTab) {
        chrome.tabs.create(request.data);
    }
});
