//namespace
var apg = {};


/**********************************
 * BEGIN User Controlled Settings
 **********************************/

apg.translateTo = 'tr';
apg.percent = 20;
apg.enabled = true;
apg.minLength = 4;
apg.highlighting = 'on';
apg.wordBlacklist = [];

//URL Filtering
apg.siteBlackList = [];
apg.siteWhiteList = [];
apg.filterType = "black";

//Dictionary Settings
apg.dictionaryGrowthRate = 4;
apg.dictionaryMaxSize = 1000;

/**********************************
 * END User Controlled Settings
 *
 * BEGIN Development Settings
 **********************************/

apg.dictionaryStubbing = false;
apg.debugging = false;

// hard limit on max words to translate, since creating too many DOM
// nodes causes a performance hit.
apg.maxWordsPerPage = 500;

//if the dictionary translates most words, we wont bother with remote requests
//TO CONSIDER: Remove this setting
apg.remoteRequestTreshold = 97;

/**********************************
 * END Development Settings
 *
 * BEGIN Properties
 **********************************/

apg.wordCount = 0;
apg.text = "";
apg.nodes = [];
apg.words = [];
apg.keptWords = [];
apg.translation = [];
apg.theText = [];

apg.dictionarySize = 0;

apg.selectedWords = [];

apg.translatedCount = 0;

apg.translateIndex = 0;

apg.trans = {};
apg.finalTrans = {};

/**
 * Main function calls the entire page translation process.
 */
apg.convert = function () {
    apg.debug('Starting Translations...');
    chrome.extension.sendMessage({getSettings : true}, function (response) {

        apg.applySettings(response);

        var enabled = apg.translateEnabled();

        if (enabled === true) {
            apg.timeStart("Full Flewent Execution");
            apg.timeStart("Full Execution Before Ajax");
            apg.catchClicks();
            apg.getNodes(document.body);
            apg.getText();
            apg.shuffleKeptWords();
            apg.doTranslations();
        }
    });
};

apg.applySettings = function (response) {
    apg.translateTo = response.translateTo;
    apg.percent = response.convertPercent;
    apg.enabled = response.enabled;
    apg.minLength = response.minLength;
    apg.highlighting = response.highlighting;
    apg.wordBlackList = response.wordBlackList;
    apg.siteBlackList = response.siteBlackList;
    apg.siteWhiteList = response.siteWhiteList;
    apg.filterType = response.filterType;
    apg.dictionaryGrowthRate = response.growthRate;
    apg.dictionaryMaxSize = response.maxDictionary;
}

apg.translateEnabled = function () {
    if (apg.enabled === "false" || apg.enabled === false) {
        return false;
    }
    var enabled;
    var loc = window.location.toString();

    var lang = $('html').attr('lang');
    if (lang === undefined || lang.indexOf('en') > -1 || lang.indexOf('EN') > -1) {
        //language not set or contains `en`, its okay.
    }
    else {
        apg.userMessage('Disabling because the source language (' + lang + ') is non-english');
        return false;
    }

    if (apg.filterType == "black") {

        enabled = true;

        $.each(apg.siteBlackList, function (i, value) {
            if (loc.indexOf(value) !== -1) {
                enabled = false;
                apg.userMessage("disabling because site is in blacklist");
            }
        });


    }
    else {

        enabled = false;

        $.each(apg.siteWhiteList, function (i, value) {
            if (loc.indexOf(value) > -1) {
                enabled = true;
            }
        });

        if (enabled == false) {
            apg.userMessage("disabling because site is not in whitelist");
        }
    }

    return enabled;
};

apg.catchClicks = function () {
    $(document).click(function (event) {
        var target = $(event.target);
        var findClass = "llWordDetailPopup";

        if ($("." + findClass).length > 0) {

            if (!apg.parentHasClass(target, findClass) && !apg.parentHasClass(target, "llPopupSpan")) {
                $("." + findClass).remove();
            }
        }
    });
}

apg.parentHasClass = function (element, theClass) {
    var hasClass = false;

    //if the element itself has the class, we return true;
    if (element.hasClass(theClass)) {
        return true;
    }

    //check all the parents
    element.parents().each(function () {
        if ($(this).hasClass(theClass)) {
            hasClass = true;
        }
    });

    //return result
    return hasClass;
}

apg.tagIsAllowed = function (tagName) {
    switch (tagName) {
        case 'a':
            return false;
            break;
        case 'style':
            return false;
            break;
        case 'script':
            return false;
            break;
        case 'noscript':
            return false;
            break;
        case 'code':
            return false;
            break;
        case 'select':
            return false;
            break;
        default:
            return true;
    }
};

apg.getNodes = function (element) {
    var data;
    for (var i = 0, n = element.childNodes.length; i < n; i++) {
        var child = element.childNodes[i];
        if (child.nodeType === 1 && apg.tagIsAllowed(child.tagName.toLowerCase())) {
            apg.getNodes(child);
        }
        else if (child.nodeType === 3) {
            data = child.data.replace(/[^a-zA-Z\-]+/ig, "");
            if ($.trim(data) != "") {
                apg.nodes.push(child);
            }
        }
    }
    return;
}

apg.getText = function () {

    for (var i in apg.nodes) {
        apg.text += apg.nodes[i].textContent + " ";
    }

    apg.text = apg.text.replace(/[\s]+/ig, " ");

    apg.text = apg.text.replace(/[0-9\[\]\(\)\<\>]+/ig, "");

    apg.words = apg.text.split(" ");

    //unset text to hopefully reduce memory consumption? not sure if it helps :P
    apg.text = "";

    var myWord = "";
    var inArray = false;


    for (var i in apg.words) {
        myWord = apg.words[i].toLowerCase();

        if (myWord.length >= apg.minLength && myWord.match(/^[a-zA-Z]+$/) != null) {

            var isBlacklisted = $.inArray(myWord, apg.wordBlackList);

            if (isBlacklisted === -1) {

                inArray = $.inArray(myWord, apg.keptWords);

                if (inArray === -1) {
                    apg.keptWords.push(myWord);
                }

            }

        }
    }

    return true;
};


apg.shuffleKeptWords = function () {
    var i = apg.keptWords.length;

    if (i == 0) {
        return false;
    }

    while (--i) {
        var j = Math.floor(Math.random() * ( i + 1 ));
        var tempi = apg.keptWords[i];
        var tempj = apg.keptWords[j];
        apg.keptWords[i] = tempj;
        apg.keptWords[j] = tempi;
    }

    return true;
}

apg.getNumberToTranslate = function () {
    var multiplier = (apg.percent / 100),
        length = apg.keptWords.length,
        numberToTranslate = Math.floor(length * multiplier);

    if (numberToTranslate > apg.maxWordsPerPage) {
        return apg.maxWordsPerPage;
    }

    return numberToTranslate;


};

apg.translateFromDictionary = function () {
    apg.timeStart('Dictionary Translation');
    var translateCount = apg.getNumberToTranslate(),
        request = {
            dictionaryTranslate : true,
            toTranslate         : apg.keptWords,
            translateCount      : translateCount,
            noTranslateCount    : apg.dictionaryGrowthRate,
            currentLanguage     : apg.translateTo
        };

    if (apg.dictionarySize >= apg.dictionaryMaxSize) {
        request.noTranslateCount = 0;
    }

    chrome.extension.sendMessage(request, function (response) {
        var i, translatedWord, percent,
            totalWords = 0, fromDictionary = 0,
            newSelectedWords = [];

        for (i in response) {
            translatedWord = response[i];
            totalWords += 1;
            if (translatedWord !== false) {
                apg.finalTrans[i] = translatedWord;
                fromDictionary += 1;
            }
            else {
                newSelectedWords.push(i);
            }
        }

        percent = Math.floor((fromDictionary / totalWords) * 100);

        apg.debug(" - Wanted to translate " + translateCount + " words from Dictionary.");
        apg.debug(" - Translated " + fromDictionary + ' words from dictionary. ');
        apg.debug(" - Sending " + (totalWords - fromDictionary) + ' to server for translation.');

        apg.selectedWords = newSelectedWords;

        apg.updateDocument();

        apg.finalTrans = {};
        apg.nodes = [];
        apg.getNodes(document.body);

        apg.timeEnd('Dictionary Translation');

        apg.timeEnd("Full Execution Before Ajax");
        //if (percent < apg.remoteRequestTreshold) {
        if ((totalWords - fromDictionary) > 0) {
            apg.translateWords();
        }
        else {
            chrome.extension.sendMessage({updateIcon : true, path : 'logo_48.png'});
            apg.timeEnd('Full Flewent Execution');
        }
    });
};

apg.doTranslations = function () {
    if (apg.dictionaryStubbing === true) {
        apg.dictionaryGrowthRate = 600;
        apg.maxWordsPerPage = 600;
    }
    var getSizeRequest = {
        getDictionarySize : true,
        currentLanguage   : apg.translateTo
    };

    chrome.extension.sendMessage({updateIcon : true, path : 'logo_loading_48.png'});

    chrome.extension.sendMessage(getSizeRequest, function (size) {
        apg.dictionarySize = size;
        apg.translateFromDictionary();
    });

}

apg.translateWords = function () {
    var currentWord = apg.selectedWords[apg.translateIndex];
    var url = '//api.mymemory.translated.net/get', stubResponse;

    apg.debug('Adding '+currentWord+' to Dictionary...');
    //dictionary stubbing is used to fill a dictionary with stubbed translations
    if (apg.dictionaryStubbing === true) {
        stubResponse = {
            responseData : {
                translatedText : currentWord + '_' + apg.translateTo
            }
        }
        return apg.translateCallback(stubResponse);
    }

    if(currentWord === undefined || !currentWord) {
        return;
    }

    $.ajax({
        url      : url,
        type     : "get",
        dataType : "json",
        data     : {
            q        : currentWord,
            de       : 'andrew.p.groff+testing@gmail.com',
            //de: 'andrew.p.groff+flewent@gmail.com',
            langpair : 'en|' + apg.translateTo
        },
        success  : function (data) {
            if (data.responseStatus == 403) {
                apg.userMessage('Max daily requests reached. Aborting further requests.');
                return;
            }
            apg.translateCallback(data);
        }
    });

};

apg.translateCallback = function (data) {
    var response = data.responseData.translatedText;
    response = response.toLowerCase();
    var translated = response;

    apg.setTrans(translated);

    apg.translateIndex += 1;

    if (apg.translateIndex >= apg.selectedWords.length || apg.translateIndex == apg.dictionaryGrowthRate) {
        apg.setFinalTrans();
        apg.updateDocument();
        apg.addToDictionary();
        chrome.extension.sendMessage({updateIcon : true, path : 'logo_48.png'});
        apg.timeEnd('Full Flewent Execution');
    }
    else {
        apg.translateWords();
    }

};

apg.addToDictionary = function () {
    var message = {
        addToDictionary : true,
        currentLanguage : apg.translateTo,
        wordsToAdd      : apg.finalTrans
    };
    chrome.extension.sendMessage(message);
}

apg.setTrans = function (translated) {
    var selected;
    selected = apg.selectedWords[apg.translateIndex];

    apg.trans[selected] = translated;
    apg.translatedCount += 1;
    //console.log(translated);
}

apg.setFinalTrans = function () {
    var errors = 0;
    $.each(apg.trans, function (index, value) {
        apg.finalTrans[index] = $.trim(value);
    });

    var count = apg.translatedCount, percent;
    var difference = count - errors;
    if (count > 0) {
        percent = difference / count * 100;
        percent = Math.floor(percent);
    }
    else {
        percent = 0;
    }

    var report = "";

    report += "Translation report: \n\n";
    report += apg.selectedWords.length + " words were sent for translation. \n";
    report += count + " were translated with " + errors + " errors. \n";
    report += "This makes a success rate of ~" + percent + "%. \n";
    report += "Errored words not translated in page. ";

    //console.log(report);
    //console.log(apg.selectedWords);
}

apg.getWordsString = function () {
    var string = "", val;
    var words = apg.selectedWords;

    $.each(words, function (index, value) {
        val = $.trim(value)
        if (string.length + val.length < 1000) {
            string += val + " | ";
        }
    });

    return string.substr(0, string.length - 3);
};

apg.updateDocument = function () {
    apg.timeStart("Update Document");
    $.each(apg.nodes, function (index, node) {
        apg.replaceInNode(node);
    });
    apg.timeEnd("Update Document");
};

apg.capitalizeWord = function(word){
    return word[0].toLocaleUpperCase() + word.slice(1);
}

apg.replaceInNode = function (node) {
    var original, pos,
        firstChar,
        match, matches = new Array();
    var originalText = ' ' + node.textContent + ' ';
    var text = originalText.toLocaleLowerCase();

    $.each(apg.finalTrans, function (index, value) {
        original = index;

        if (value == "") {
            return;
        }

        pos = text.indexOf(' ' + original + ' ');

        if (pos !== -1) {
            firstChar = originalText.substr((pos+1), 1);

            if(firstChar.toLocaleLowerCase() != firstChar) {
                value = apg.capitalizeWord(value);
            }
            match = {
                'original'    : original,
                'translation' : value,
                'start'       : pos,
                'end'         : (pos + original.length)
            };
            matches.push(match);
        }

    });

    if (matches.length > 0) {
        matches.sort(function (a, b) {
            return a.start - b.start;
        });

        apg.applyMatches(node, matches);
    }
};

apg.applyMatches = function (node, matches) {
    var pos, match, ii, partialText;
    var parent = node.parentNode;
    var fragment = document.createDocumentFragment();
    var text = node.textContent;
    var from = 0;

    for (var i in matches) {
        //ii represents the integer version of i for later math
        ii = parseInt(i, 10);
        match = matches[i]
        pos = match.start;

        //if there was a match before this, start at that character
        if (matches[ii - 1]) {
            from = matches[ii - 1].end;
        }

        if (pos !== 0) {
            //get the text from the end of the last match (or beginning of string)
            //until the beginning of this match
            partialText = text.substr(from, (pos - from));

            //append node
            fragment.appendChild(document.createTextNode(' ' + partialText + ' '));
        }

        //append the translated span
        fragment.appendChild(apg.getSpan(match.original, match.translation));

        //if no  matches after this one
        if (!matches[ii + 1]) {
            //if this match doesnt run to the end of the string
            if ((pos + match.original.length + 1) < text.length) {
                //get the rest of the string
                partialText = text.substr(pos + match.original.length + 1);

                //append it
                fragment.appendChild(document.createTextNode(' ' + partialText + ' '));
            }
        }
        fragment.appendChild(document.createTextNode(' '));
    }

    //replace text node with new series of nodes.
    parent.replaceChild(fragment, node);
}

apg.getSpan = function (original, translation) {
    //create span
    var e = document.createElement("span");

    if ($.trim(translation) == $.trim(original)) {
        e.title = "This word was not translated, or is the same in both languages";
        e.style.backgroundColor = "#FDFDFD";
        e.style.color = '#666';
        //display the original text
        translation = original;
    }
    else {
        //set properties
        e.style.backgroundColor = "#FFFFCA";
        e.style.color = '#222';
        e.title = "Original text: " + original;
    }

    if (apg.highlighting === 'off') {
        e.style.backgroundColor = 'inherit';
    }

    e.className = "llPopupSpan"
    e.style.display = 'inline';
    e.style.cursor = 'pointer';
    e.style.fontSize = 'inherit';
    e.onclick = function (e) {
        apg.translationClicked(original, translation, e);
    };

    //append its text
    e.appendChild(document.createTextNode(translation));

    return e;
};

apg.translationClicked = function (original, translation, e) {
    // close all:
    $('.llWordDetailPopup').remove();

    var span = $(e.target);
    var pos = span.offset();
    var height = span.height();
    var div = $('<div></div>');

    div.addClass('llWordDetailPopup');

    var css = apg.css("div");
    css.top = pos.top + height + 5
    css.left = pos.left;

    div.css(css);

    var html = "";

    var template = {
        getTemplate  : true,
        variables    : {
            original    : original,
            translation : translation,
            translateTo : apg.translateTo
        },
        templateName : 'popup_ui'
    };
    chrome.extension.sendMessage(template, function (html) {
        div.append(html);

        div = apg.applyPopupCss(div);
        div = apg.applyPopupActions(div, translation, original);

        $('body').append(div);
    });

};

apg.applyPopupCss = function (div) {
    $('a', div).css({color : '#000'});
    $('.mainWord', div).css(apg.css('mainWord'));
    $('.attribution', div).css(apg.css('attribution'));
    $('.moreOptions', div).css(apg.css('moreOptions'));
    $('.llListen', div).css(apg.css('llListen'));
    $('.flewentButton', div).css(apg.css('flewentButton'));
    $('.saveEditButton', div).css({width : '60px'});

    $('.moreOptions', div).hide();

    $('.hoverState', div).hover(
        function () {
            $(this).css('background-color', '#AAA');
        },
        function () {
            $(this).css('background-color', '#CCC');
        }
    );

    return div;
};

apg.applyPopupActions = function (div, translation, original) {


    $('.dictionary', div).click(function () {
        var message = {
            newTab : true,
            data   : {url : "dictionary.html"}
        };
        chrome.extension.sendMessage(message);
    });

    $('.llListen', div).click(function () {
        var frame = apg.getListenFrame(translation);
        frame.prependTo(div);
    });

    $('.editLink', div).click(function () {
        $('.editContainer', div).toggle();
    });
    $('.moreLink', div).click(function () {
        $('.moreOptions', div).toggle();
    });

    $('.llSucks', div).click(function () {
        apg.openAboutPage('#itsucks');
    });

    $('.llRich', div).click(function () {
        apg.openAboutPage('#imrich');
    });

    $('.deleteButton', div).click(function () {
        var message = {
                removeFromDictionary : true,
                currentLanguage      : apg.translateTo,
                toRemove             : original
            };

        chrome.extension.sendMessage(message);

        $('.mainContents', div).hide();
        div.append('<span> Dictionary Updated. <br />`' + original + '` was removed.`</span>');

        $('.llPopupSpan').each(function () {
            if ($(this).html() == translation) {
                $(this).after('<span>'+original+'</span>');
                $(this).remove();
            }
        })
    });

    $('.saveEditButton', div).click(function () {
        var message = {
                addToDictionary : true,
                currentLanguage : apg.translateTo,
                force           : true,
                wordsToAdd      : {}
            },
            newValue = $('.newEditValue', div).val();

        message.wordsToAdd[original] = newValue;

        chrome.extension.sendMessage(message);

        $('.mainContents', div).hide();
        div.append('<span> Dictionary Updated. <br />`' + original + '` was changed to `' + newValue + '`</span>');

        $('.llPopupSpan').each(function () {
            if ($(this).html() == translation) {
                $(this).html(newValue);
            }
        })
    });

    return div;
};

apg.getListenFrame = function (translation) {
    var src = 'http://translate.google.com/translate_tts?ie=UTF-8&q='
        + translation + '&tl='
        + apg.translateTo
        + '&total=1&idx=0&textlen=' + translation.length;
    var frame = $('<iframe></iframe>');
    frame.attr("src", src);
    frame.hide();
    return frame;
}

apg.openAboutPage = function (hash) {
    var theHash = hash || "";
    var message = {
        newTab : true,
        data   : {url : "about.html" + theHash}
    };
    chrome.extension.sendMessage(message);
};

apg.css = function (element) {
    if (element == "div") {
        return {
            'z-index'          : '600',
            'font-family'      : 'Ubuntu, Arial, sans-serif',
            'border'           : '1px solid #303942',
            'border-radius'    : '5px',
            'text-align'       : 'left',
            'color'            : '#303942',
            'background-color' : '#FDFDFD',
            'font-size'        : '13px',
            'padding'          : '14px',
            'width'            : '340px',
            'position'         : 'absolute'
        };
    }

    if (element == "flewentButton") {
        return {
            'display'          : 'inline-block',
            'margin'           : '2px',
            'width'            : '150px',
            'text-align'       : 'center',
            'background-color' : '#CCC',
            'border-radius'    : '5px',
            'text-decoration'  : 'none',
            'padding'          : '3px 5px'
        };
    }
//    if (element == "dictionary") {
//        return {
//            'background-color' : '#FFFFFF',
//            'float'            : 'right',
//            'border-radius'    : '4px',
//            'border'           : '1px solid #303942',
//            'cursor'           : 'pointer',
//            'width'            : '24px',
//            'padding'          : '0px 5px'
//        };
//    }

    if (element == "llListen") {
        var url = 'chrome-extension://' + chrome.i18n.getMessage("@@extension_id") + "/ui-icons.png";
        return {
            'background-color'    : '#FFFFFF',
            'background-image'    : 'url(' + url + ')',
            'background-position' : '-142px -161px',
            'float'               : 'left',
            'width'               : '20px',
            'height'              : '15px',
            'border-radius'       : '4px',
            'border'              : '1px solid #303942',
            'margin-right'        : '6px',
            'cursor'              : 'pointer'
        };
    }

    if (element == "moreOptions") {
        return {
            'border-radius' : '4px',
            'padding'       : '11px',
            'border'        : '1px solid #CCC',
            'margin'        : '8px 0px'
        };
    }

    if (element == "mainWord") {
        return {
            'text-transform' : 'capitalize',
            'font-size'      : '14px',
            'color'          : '#000'
        };
    }

    if (element == "attribution") {
        return {
            'font-size'  : '12px',
            'font-style' : 'italic'
        };
    }

    return {};
}

/*************************************
 * Debugging / Output Methods
 *************************************/
apg.timeStart = function (label) {
    if (!apg.debugging) {return;}
    console.time(label);
}
apg.timeEnd = function (label) {
    if (!apg.debugging) {return;}
    console.timeEnd(label);
}
apg.debug = function () {
    if (!apg.debugging) {return;}
    for (i in arguments) {
        console.log(arguments[i]);
    }
}
apg.userMessage = function () {
    for (i in arguments) {
        console.log(arguments[i]);
    }
};

$(function () {
    apg.convert();
});
